
#include "simpletools.h"                      // Include simple tools
#include "stdio.h"

/*=====================================================================================================*/
/* Display numbers in PPDB 16-segment LED display                                                      */
/*=====================================================================================================*/
volatile unsigned char led_nums[6] = { 0,0,0,0,0,0 };

/*
 * Pin assignments:
 * 
 * A1-A2: 6
 * B: 7
 * C: 8
 * D1-D2: 9
 * E: 10
 * F: 11
 * G1-G2: 12
 *
 *      6
 *    +----+
 *  11| 12 |7
 *    +----+
 *  10|    |8
 *    +----+
 *      9
 *      
 * Jumper pins A1 and A2, D1 and D2, and G1 and G2 (to simulate a 7-segment display). 
 *      
 * L Dig 1: 5
 * R Dig 1: 4
 * L Dig 2: 3
 * R Dig 2: 2
 * L Dig 3: 1
 * R Dig 3: 0
 * 
 */

// Run this in a cog at startup. 
void numdisplay() {
  unsigned char dindex[6] = { 5,4,3,2,1,0 };

  set_outputs(12,0,0);
  set_outputs(5,0,1);
  set_directions(12,0,0x1fff);

  while (1) {
    for (int i=0; i < 6; i++) {
      set_outputs(12,6,led_nums[i]);
      set_output(dindex[i],0);
      pause(1);
      set_output(dindex[i],1);
    }
    pause(4);
  }
}

// Call this to set the number to display.
void setnum(int n) {
  char buf[7];
  int leadingZeros = 1;
  //                                        0     1     2     3     4     5     6     7     8     9    10
  const unsigned char led_digits[11] = {0x3f, 0x06, 0x5b, 0xcf, 0x66, 0x6d, 0x7d, 0x07, 0x7f, 0x6f, 0x00 };
  
  sprintf(buf, "%06d", n);
  for (int i=0; i < 6; i++) {
     int j = buf[i] - 0x30; // decimal ASCII to binary
     if (j == 0 && leadingZeros && i < 5) led_nums[i] = 0;
     else {
      led_nums[i] = led_digits[j];
      if (leadingZeros && j != 0) leadingZeros = 0;
     }     
  }      
}  

/*=====================================================================================================*/
/* RTC functions                                                                                       */
/*=====================================================================================================*/

// RTC pin assignments
int rtc_clk = 25;
int rtc_io = 24;
int rtc_ce = 23;


unsigned char rtc_command(unsigned char clock_or_ram, unsigned char reg, unsigned char w_or_r) {
  return (1<<7) + (clock_or_ram<<6) + (reg<<1) + w_or_r;
}  

void rtc_writeByte(unsigned char cmd) {
  set_direction(rtc_io,1);
  for (int i=0; i < 8; i++) {
     int bit = cmd & 1;
     set_output(rtc_io,bit);
     set_output(rtc_clk,1);
     cmd = cmd >> 1;
     set_output(rtc_clk,0);
  }    
}  

unsigned char rtc_readByte() {
  unsigned char retval = 0;
  for(int i=0; i < 8; i++) {
     if (get_state(rtc_io) == 1) {
       retval += (1 << i);
     }
     set_output(rtc_clk,1);
     set_output(rtc_clk,0);
  }
  return retval;
}  

unsigned char rtc_read(unsigned char cmd) {
   set_output(rtc_ce,1);
   rtc_writeByte(cmd);
   set_direction(rtc_io,0);
   unsigned char b = rtc_readByte();
   set_output(rtc_ce,0);
   return b;
}  

void rtc_write(unsigned char cmd, unsigned char data) {
  set_output(rtc_ce,1);
  rtc_writeByte(cmd);
  rtc_writeByte(data);
  set_output(rtc_ce,0);  
}  

unsigned char bin2bcd(unsigned char bin) {
   unsigned char tmp = bin/10;
   return bin - (tmp * 10) + (tmp << 4);
}   

unsigned char bcd2bin(unsigned char bcd) {
  return (bcd & 0x0f) + ((bcd >> 4) & 0x0f)*10;
}  

typedef struct {
  unsigned char yr;
  unsigned char mo;
  unsigned char day;
  unsigned char dow;
  unsigned char hr;
  unsigned char min;
  unsigned char sec;
} DATETIME;

DATETIME rtc_readDateTime() {
  DATETIME retval;
  retval.yr = rtc_read(0x8d);
  retval.mo = rtc_read(0x89);
  retval.day = rtc_read(0x87);
  retval.dow = rtc_read(0x8b);

  retval.hr = rtc_read(0x85);
  retval.min = rtc_read(0x83);
  retval.sec = rtc_read(0x81);
  return retval;
}

void rtc_setDatetime(DATETIME dt) {
  rtc_write(0x8c, bin2bcd(dt.yr));
  rtc_write(0x8a, dt.dow);
  rtc_write(0x88, bin2bcd(dt.mo));
  rtc_write(0x86, bin2bcd(dt.day));
  rtc_write(0x84, bin2bcd(dt.hr));
  rtc_write(0x82, bin2bcd(dt.min));
  rtc_write(0x80, bin2bcd(dt.sec));
}  


void rtc_init() {
  set_direction(rtc_ce,1);
  set_output(rtc_ce, 0);
  set_direction(rtc_clk,1);
  set_output(rtc_clk,0); 
}

void rtc_config() {
  unsigned char clock = 0;
  unsigned char ram = 1;
  unsigned char w = 0;
  unsigned char r = 1;
  unsigned char sec = 0;
  unsigned char mi = 1;
  unsigned char hr = 2;
  unsigned char day = 3;
  unsigned char mo = 4;
  unsigned char dow = 5;
  unsigned char yr = 6;
  unsigned char ctrl = 7;
  unsigned char tc = 8;
  unsigned char burst = 31;
  
  
  // Config RTC (once after power-up)
  rtc_write(rtc_command(clock,ctrl,w),0);                                       //clear write-protect bit
  rtc_write(rtc_command(clock,sec,w),rtc_read(rtc_command(clock,sec,r)) & 0x7f);//clear halt bit
  rtc_write(rtc_command(clock,hr,w),rtc_read(rtc_command(clock,hr,r)) & 0x7f);  //set 24hr mode
  rtc_write(rtc_command(clock,tc,w),0);                                         //disable trickle charger
}  

/*=====================================================================================================*/
/* Main program                                                                                        */
/*=====================================================================================================*/

int main()
{
  // Run the LED numeric display process.
  cog_run(numdisplay,128);

  // Initialize the RTC.
  rtc_init();
  rtc_config();
  DATETIME dt = { 0,0,0,0,0,0,0 };
  rtc_setDatetime(dt);
  
  // Show elapsed time in the LED numeric display.
  while (1) {
    DATETIME t = rtc_readDateTime();
    int num = (bcd2bin(t.hr) * 10000) + (bcd2bin(t.min) * 100) + bcd2bin(t.sec);
    setnum(num);
    pause(10);
  }    
}
